@@ -685,6 +685,39 @@ namespace Model
     inline Branch& WithEnableBasicAuth(bool value) { SetEnableBasicAuth(value); return *this;}
 
 
+    /**
+     * <p>Enables performance mode for the branch.</p> <p>Performance mode optimizes
+     * for faster hosting performance by keeping content cached at the edge for a
+     * longer interval. When performance mode is enabled, hosting configuration or code
+     * changes can take up to 10 minutes to roll out. </p>
+     */
+    inline bool GetEnablePerformanceMode() const{ return m_enablePerformanceMode; }
+
+    /**
+     * <p>Enables performance mode for the branch.</p> <p>Performance mode optimizes
+     * for faster hosting performance by keeping content cached at the edge for a
+     * longer interval. When performance mode is enabled, hosting configuration or code
+     * changes can take up to 10 minutes to roll out. </p>
+     */
+    inline bool EnablePerformanceModeHasBeenSet() const { return m_enablePerformanceModeHasBeenSet; }
+
+    /**
+     * <p>Enables performance mode for the branch.</p> <p>Performance mode optimizes
+     * for faster hosting performance by keeping content cached at the edge for a
+     * longer interval. When performance mode is enabled, hosting configuration or code
+     * changes can take up to 10 minutes to roll out. </p>
+     */
+    inline void SetEnablePerformanceMode(bool value) { m_enablePerformanceModeHasBeenSet = true; m_enablePerformanceMode = value; }
+
+    /**
+     * <p>Enables performance mode for the branch.</p> <p>Performance mode optimizes
+     * for faster hosting performance by keeping content cached at the edge for a
+     * longer interval. When performance mode is enabled, hosting configuration or code
+     * changes can take up to 10 minutes to roll out. </p>
+     */
+    inline Branch& WithEnablePerformanceMode(bool value) { SetEnablePerformanceMode(value); return *this;}
+
+
     /**
      * <p> The thumbnail URL for the branch of an Amplify app. </p>
      */
@@ -904,22 +937,22 @@ namespace Model
 
 
     /**
-     * <p> Enables pull request preview for the branch. </p>
+     * <p> Enables pull request previews for the branch. </p>
      */
     inline bool GetEnablePullRequestPreview() const{ return m_enablePullRequestPreview; }
 
     /**
-     * <p> Enables pull request preview for the branch. </p>
+     * <p> Enables pull request previews for the branch. </p>
      */
     inline bool EnablePullRequestPreviewHasBeenSet() const { return m_enablePullRequestPreviewHasBeenSet; }
 
     /**
-     * <p> Enables pull request preview for the branch. </p>
+     * <p> Enables pull request previews for the branch. </p>
      */
     inline void SetEnablePullRequestPreview(bool value) { m_enablePullRequestPreviewHasBeenSet = true; m_enablePullRequestPreview = value; }
 
     /**
-     * <p> Enables pull request preview for the branch. </p>
+     * <p> Enables pull request previews for the branch. </p>
      */
     inline Branch& WithEnablePullRequestPreview(bool value) { SetEnablePullRequestPreview(value); return *this;}
 
@@ -1145,6 +1178,9 @@ namespace Model
     bool m_enableBasicAuth;
     bool m_enableBasicAuthHasBeenSet;
 
+    bool m_enablePerformanceMode;
+    bool m_enablePerformanceModeHasBeenSet;
+
     Aws::String m_thumbnailUrl;
     bool m_thumbnailUrlHasBeenSet;
 
