@@ -42,6 +42,7 @@ App::App() :
     m_customRulesHasBeenSet(false),
     m_productionBranchHasBeenSet(false),
     m_buildSpecHasBeenSet(false),
+    m_customHeadersHasBeenSet(false),
     m_enableAutoBranchCreation(false),
     m_enableAutoBranchCreationHasBeenSet(false),
     m_autoBranchCreationPatternsHasBeenSet(false),
@@ -73,6 +74,7 @@ App::App(JsonView jsonValue) :
     m_customRulesHasBeenSet(false),
     m_productionBranchHasBeenSet(false),
     m_buildSpecHasBeenSet(false),
+    m_customHeadersHasBeenSet(false),
     m_enableAutoBranchCreation(false),
     m_enableAutoBranchCreationHasBeenSet(false),
     m_autoBranchCreationPatternsHasBeenSet(false),
@@ -225,6 +227,13 @@ App& App::operator =(JsonView jsonValue)
     m_buildSpecHasBeenSet = true;
   }
 
+  if(jsonValue.ValueExists("customHeaders"))
+  {
+    m_customHeaders = jsonValue.GetString("customHeaders");
+
+    m_customHeadersHasBeenSet = true;
+  }
+
   if(jsonValue.ValueExists("enableAutoBranchCreation"))
   {
     m_enableAutoBranchCreation = jsonValue.GetBool("enableAutoBranchCreation");
@@ -382,6 +391,12 @@ JsonValue App::Jsonize() const
 
   }
 
+  if(m_customHeadersHasBeenSet)
+  {
+   payload.WithString("customHeaders", m_customHeaders);
+
+  }
+
   if(m_enableAutoBranchCreationHasBeenSet)
   {
    payload.WithBool("enableAutoBranchCreation", m_enableAutoBranchCreation);
