@@ -1,17 +1,7 @@
-﻿/*
-* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
-*
-* Licensed under the Apache License, Version 2.0 (the "License").
-* You may not use this file except in compliance with the License.
-* A copy of the License is located at
-*
-*  http://aws.amazon.com/apache2.0
-*
-* or in the "license" file accompanying this file. This file is distributed
-* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
-* express or implied. See the License for the specific language governing
-* permissions and limitations under the License.
-*/
+﻿/**
+ * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
+ * SPDX-License-Identifier: Apache-2.0.
+ */
 
 #pragma once
 #include <aws/batch/Batch_EXPORTS.h>
@@ -35,12 +25,16 @@ namespace Model
 {
 
   /**
+   * <p>Contains the parameters for <code>SubmitJob</code>.</p><p><h3>See Also:</h3> 
+   * <a
+   * href="http://docs.aws.amazon.com/goto/WebAPI/batch-2016-08-10/SubmitJobRequest">AWS
+   * API Reference</a></p>
    */
   class AWS_BATCH_API SubmitJobRequest : public BatchRequest
   {
   public:
     SubmitJobRequest();
-    
+
     // Service request name is the Operation name which will send this request out,
     // each operation should has unique request name, so that we can get operation's name from this request.
     // Note: this is not true for response, multiple operations may have the same response name,
@@ -53,105 +47,105 @@ namespace Model
     /**
      * <p>The name of the job. The first character must be alphanumeric, and up to 128
      * letters (uppercase and lowercase), numbers, hyphens, and underscores are
-     * allowed. </p>
+     * allowed.</p>
      */
     inline const Aws::String& GetJobName() const{ return m_jobName; }
 
     /**
      * <p>The name of the job. The first character must be alphanumeric, and up to 128
      * letters (uppercase and lowercase), numbers, hyphens, and underscores are
-     * allowed. </p>
+     * allowed.</p>
      */
     inline bool JobNameHasBeenSet() const { return m_jobNameHasBeenSet; }
 
     /**
      * <p>The name of the job. The first character must be alphanumeric, and up to 128
      * letters (uppercase and lowercase), numbers, hyphens, and underscores are
-     * allowed. </p>
+     * allowed.</p>
      */
     inline void SetJobName(const Aws::String& value) { m_jobNameHasBeenSet = true; m_jobName = value; }
 
     /**
      * <p>The name of the job. The first character must be alphanumeric, and up to 128
      * letters (uppercase and lowercase), numbers, hyphens, and underscores are
-     * allowed. </p>
+     * allowed.</p>
      */
     inline void SetJobName(Aws::String&& value) { m_jobNameHasBeenSet = true; m_jobName = std::move(value); }
 
     /**
      * <p>The name of the job. The first character must be alphanumeric, and up to 128
      * letters (uppercase and lowercase), numbers, hyphens, and underscores are
-     * allowed. </p>
+     * allowed.</p>
      */
     inline void SetJobName(const char* value) { m_jobNameHasBeenSet = true; m_jobName.assign(value); }
 
     /**
      * <p>The name of the job. The first character must be alphanumeric, and up to 128
      * letters (uppercase and lowercase), numbers, hyphens, and underscores are
-     * allowed. </p>
+     * allowed.</p>
      */
     inline SubmitJobRequest& WithJobName(const Aws::String& value) { SetJobName(value); return *this;}
 
     /**
      * <p>The name of the job. The first character must be alphanumeric, and up to 128
      * letters (uppercase and lowercase), numbers, hyphens, and underscores are
-     * allowed. </p>
+     * allowed.</p>
      */
     inline SubmitJobRequest& WithJobName(Aws::String&& value) { SetJobName(std::move(value)); return *this;}
 
     /**
      * <p>The name of the job. The first character must be alphanumeric, and up to 128
      * letters (uppercase and lowercase), numbers, hyphens, and underscores are
-     * allowed. </p>
+     * allowed.</p>
      */
     inline SubmitJobRequest& WithJobName(const char* value) { SetJobName(value); return *this;}
 
 
     /**
-     * <p>The job queue into which the job is submitted. You can specify either the
-     * name or the Amazon Resource Name (ARN) of the queue. </p>
+     * <p>The job queue where the job is submitted. You can specify either the name or
+     * the Amazon Resource Name (ARN) of the queue.</p>
      */
     inline const Aws::String& GetJobQueue() const{ return m_jobQueue; }
 
     /**
-     * <p>The job queue into which the job is submitted. You can specify either the
-     * name or the Amazon Resource Name (ARN) of the queue. </p>
+     * <p>The job queue where the job is submitted. You can specify either the name or
+     * the Amazon Resource Name (ARN) of the queue.</p>
      */
     inline bool JobQueueHasBeenSet() const { return m_jobQueueHasBeenSet; }
 
     /**
-     * <p>The job queue into which the job is submitted. You can specify either the
-     * name or the Amazon Resource Name (ARN) of the queue. </p>
+     * <p>The job queue where the job is submitted. You can specify either the name or
+     * the Amazon Resource Name (ARN) of the queue.</p>
      */
     inline void SetJobQueue(const Aws::String& value) { m_jobQueueHasBeenSet = true; m_jobQueue = value; }
 
     /**
-     * <p>The job queue into which the job is submitted. You can specify either the
-     * name or the Amazon Resource Name (ARN) of the queue. </p>
+     * <p>The job queue where the job is submitted. You can specify either the name or
+     * the Amazon Resource Name (ARN) of the queue.</p>
      */
     inline void SetJobQueue(Aws::String&& value) { m_jobQueueHasBeenSet = true; m_jobQueue = std::move(value); }
 
     /**
-     * <p>The job queue into which the job is submitted. You can specify either the
-     * name or the Amazon Resource Name (ARN) of the queue. </p>
+     * <p>The job queue where the job is submitted. You can specify either the name or
+     * the Amazon Resource Name (ARN) of the queue.</p>
      */
     inline void SetJobQueue(const char* value) { m_jobQueueHasBeenSet = true; m_jobQueue.assign(value); }
 
     /**
-     * <p>The job queue into which the job is submitted. You can specify either the
-     * name or the Amazon Resource Name (ARN) of the queue. </p>
+     * <p>The job queue where the job is submitted. You can specify either the name or
+     * the Amazon Resource Name (ARN) of the queue.</p>
      */
     inline SubmitJobRequest& WithJobQueue(const Aws::String& value) { SetJobQueue(value); return *this;}
 
     /**
-     * <p>The job queue into which the job is submitted. You can specify either the
-     * name or the Amazon Resource Name (ARN) of the queue. </p>
+     * <p>The job queue where the job is submitted. You can specify either the name or
+     * the Amazon Resource Name (ARN) of the queue.</p>
      */
     inline SubmitJobRequest& WithJobQueue(Aws::String&& value) { SetJobQueue(std::move(value)); return *this;}
 
     /**
-     * <p>The job queue into which the job is submitted. You can specify either the
-     * name or the Amazon Resource Name (ARN) of the queue. </p>
+     * <p>The job queue where the job is submitted. You can specify either the name or
+     * the Amazon Resource Name (ARN) of the queue.</p>
      */
     inline SubmitJobRequest& WithJobQueue(const char* value) { SetJobQueue(value); return *this;}
 
@@ -301,58 +295,66 @@ namespace Model
 
 
     /**
-     * <p>The job definition used by this job. This value can be either a
-     * <code>name:revision</code> or the Amazon Resource Name (ARN) for the job
-     * definition.</p>
+     * <p>The job definition used by this job. This value can be one of
+     * <code>name</code>, <code>name:revision</code>, or the Amazon Resource Name (ARN)
+     * for the job definition. If <code>name</code> is specified without a revision
+     * then the latest active revision is used.</p>
      */
     inline const Aws::String& GetJobDefinition() const{ return m_jobDefinition; }
 
     /**
-     * <p>The job definition used by this job. This value can be either a
-     * <code>name:revision</code> or the Amazon Resource Name (ARN) for the job
-     * definition.</p>
+     * <p>The job definition used by this job. This value can be one of
+     * <code>name</code>, <code>name:revision</code>, or the Amazon Resource Name (ARN)
+     * for the job definition. If <code>name</code> is specified without a revision
+     * then the latest active revision is used.</p>
      */
     inline bool JobDefinitionHasBeenSet() const { return m_jobDefinitionHasBeenSet; }
 
     /**
-     * <p>The job definition used by this job. This value can be either a
-     * <code>name:revision</code> or the Amazon Resource Name (ARN) for the job
-     * definition.</p>
+     * <p>The job definition used by this job. This value can be one of
+     * <code>name</code>, <code>name:revision</code>, or the Amazon Resource Name (ARN)
+     * for the job definition. If <code>name</code> is specified without a revision
+     * then the latest active revision is used.</p>
      */
     inline void SetJobDefinition(const Aws::String& value) { m_jobDefinitionHasBeenSet = true; m_jobDefinition = value; }
 
     /**
-     * <p>The job definition used by this job. This value can be either a
-     * <code>name:revision</code> or the Amazon Resource Name (ARN) for the job
-     * definition.</p>
+     * <p>The job definition used by this job. This value can be one of
+     * <code>name</code>, <code>name:revision</code>, or the Amazon Resource Name (ARN)
+     * for the job definition. If <code>name</code> is specified without a revision
+     * then the latest active revision is used.</p>
      */
     inline void SetJobDefinition(Aws::String&& value) { m_jobDefinitionHasBeenSet = true; m_jobDefinition = std::move(value); }
 
     /**
-     * <p>The job definition used by this job. This value can be either a
-     * <code>name:revision</code> or the Amazon Resource Name (ARN) for the job
-     * definition.</p>
+     * <p>The job definition used by this job. This value can be one of
+     * <code>name</code>, <code>name:revision</code>, or the Amazon Resource Name (ARN)
+     * for the job definition. If <code>name</code> is specified without a revision
+     * then the latest active revision is used.</p>
      */
     inline void SetJobDefinition(const char* value) { m_jobDefinitionHasBeenSet = true; m_jobDefinition.assign(value); }
 
     /**
-     * <p>The job definition used by this job. This value can be either a
-     * <code>name:revision</code> or the Amazon Resource Name (ARN) for the job
-     * definition.</p>
+     * <p>The job definition used by this job. This value can be one of
+     * <code>name</code>, <code>name:revision</code>, or the Amazon Resource Name (ARN)
+     * for the job definition. If <code>name</code> is specified without a revision
+     * then the latest active revision is used.</p>
      */
     inline SubmitJobRequest& WithJobDefinition(const Aws::String& value) { SetJobDefinition(value); return *this;}
 
     /**
-     * <p>The job definition used by this job. This value can be either a
-     * <code>name:revision</code> or the Amazon Resource Name (ARN) for the job
-     * definition.</p>
+     * <p>The job definition used by this job. This value can be one of
+     * <code>name</code>, <code>name:revision</code>, or the Amazon Resource Name (ARN)
+     * for the job definition. If <code>name</code> is specified without a revision
+     * then the latest active revision is used.</p>
      */
     inline SubmitJobRequest& WithJobDefinition(Aws::String&& value) { SetJobDefinition(std::move(value)); return *this;}
 
     /**
-     * <p>The job definition used by this job. This value can be either a
-     * <code>name:revision</code> or the Amazon Resource Name (ARN) for the job
-     * definition.</p>
+     * <p>The job definition used by this job. This value can be one of
+     * <code>name</code>, <code>name:revision</code>, or the Amazon Resource Name (ARN)
+     * for the job definition. If <code>name</code> is specified without a revision
+     * then the latest active revision is used.</p>
      */
     inline SubmitJobRequest& WithJobDefinition(const char* value) { SetJobDefinition(value); return *this;}
 
@@ -463,105 +465,111 @@ namespace Model
 
 
     /**
-     * <p>A list of container overrides in JSON format that specify the name of a
+     * <p>A list of container overrides in the JSON format that specify the name of a
      * container in the specified job definition and the overrides it should receive.
-     * You can override the default command for a container (that is specified in the
-     * job definition or the Docker image) with a <code>command</code> override. You
-     * can also override existing environment variables (that are specified in the job
-     * definition or Docker image) on a container or add new environment variables to
-     * it with an <code>environment</code> override.</p>
+     * You can override the default command for a container, which is specified in the
+     * job definition or the Docker image, with a <code>command</code> override. You
+     * can also override existing environment variables on a container or add new
+     * environment variables to it with an <code>environment</code> override.</p>
      */
     inline const ContainerOverrides& GetContainerOverrides() const{ return m_containerOverrides; }
 
     /**
-     * <p>A list of container overrides in JSON format that specify the name of a
+     * <p>A list of container overrides in the JSON format that specify the name of a
      * container in the specified job definition and the overrides it should receive.
-     * You can override the default command for a container (that is specified in the
-     * job definition or the Docker image) with a <code>command</code> override. You
-     * can also override existing environment variables (that are specified in the job
-     * definition or Docker image) on a container or add new environment variables to
-     * it with an <code>environment</code> override.</p>
+     * You can override the default command for a container, which is specified in the
+     * job definition or the Docker image, with a <code>command</code> override. You
+     * can also override existing environment variables on a container or add new
+     * environment variables to it with an <code>environment</code> override.</p>
      */
     inline bool ContainerOverridesHasBeenSet() const { return m_containerOverridesHasBeenSet; }
 
     /**
-     * <p>A list of container overrides in JSON format that specify the name of a
+     * <p>A list of container overrides in the JSON format that specify the name of a
      * container in the specified job definition and the overrides it should receive.
-     * You can override the default command for a container (that is specified in the
-     * job definition or the Docker image) with a <code>command</code> override. You
-     * can also override existing environment variables (that are specified in the job
-     * definition or Docker image) on a container or add new environment variables to
-     * it with an <code>environment</code> override.</p>
+     * You can override the default command for a container, which is specified in the
+     * job definition or the Docker image, with a <code>command</code> override. You
+     * can also override existing environment variables on a container or add new
+     * environment variables to it with an <code>environment</code> override.</p>
      */
     inline void SetContainerOverrides(const ContainerOverrides& value) { m_containerOverridesHasBeenSet = true; m_containerOverrides = value; }
 
     /**
-     * <p>A list of container overrides in JSON format that specify the name of a
+     * <p>A list of container overrides in the JSON format that specify the name of a
      * container in the specified job definition and the overrides it should receive.
-     * You can override the default command for a container (that is specified in the
-     * job definition or the Docker image) with a <code>command</code> override. You
-     * can also override existing environment variables (that are specified in the job
-     * definition or Docker image) on a container or add new environment variables to
-     * it with an <code>environment</code> override.</p>
+     * You can override the default command for a container, which is specified in the
+     * job definition or the Docker image, with a <code>command</code> override. You
+     * can also override existing environment variables on a container or add new
+     * environment variables to it with an <code>environment</code> override.</p>
      */
     inline void SetContainerOverrides(ContainerOverrides&& value) { m_containerOverridesHasBeenSet = true; m_containerOverrides = std::move(value); }
 
     /**
-     * <p>A list of container overrides in JSON format that specify the name of a
+     * <p>A list of container overrides in the JSON format that specify the name of a
      * container in the specified job definition and the overrides it should receive.
-     * You can override the default command for a container (that is specified in the
-     * job definition or the Docker image) with a <code>command</code> override. You
-     * can also override existing environment variables (that are specified in the job
-     * definition or Docker image) on a container or add new environment variables to
-     * it with an <code>environment</code> override.</p>
+     * You can override the default command for a container, which is specified in the
+     * job definition or the Docker image, with a <code>command</code> override. You
+     * can also override existing environment variables on a container or add new
+     * environment variables to it with an <code>environment</code> override.</p>
      */
     inline SubmitJobRequest& WithContainerOverrides(const ContainerOverrides& value) { SetContainerOverrides(value); return *this;}
 
     /**
-     * <p>A list of container overrides in JSON format that specify the name of a
+     * <p>A list of container overrides in the JSON format that specify the name of a
      * container in the specified job definition and the overrides it should receive.
-     * You can override the default command for a container (that is specified in the
-     * job definition or the Docker image) with a <code>command</code> override. You
-     * can also override existing environment variables (that are specified in the job
-     * definition or Docker image) on a container or add new environment variables to
-     * it with an <code>environment</code> override.</p>
+     * You can override the default command for a container, which is specified in the
+     * job definition or the Docker image, with a <code>command</code> override. You
+     * can also override existing environment variables on a container or add new
+     * environment variables to it with an <code>environment</code> override.</p>
      */
     inline SubmitJobRequest& WithContainerOverrides(ContainerOverrides&& value) { SetContainerOverrides(std::move(value)); return *this;}
 
 
     /**
      * <p>A list of node overrides in JSON format that specify the node range to target
-     * and the container overrides for that node range.</p>
+     * and the container overrides for that node range.</p>  <p>This parameter
+     * isn't applicable to jobs running on Fargate resources; use
+     * <code>containerOverrides</code> instead.</p> 
      */
     inline const NodeOverrides& GetNodeOverrides() const{ return m_nodeOverrides; }
 
     /**
      * <p>A list of node overrides in JSON format that specify the node range to target
-     * and the container overrides for that node range.</p>
+     * and the container overrides for that node range.</p>  <p>This parameter
+     * isn't applicable to jobs running on Fargate resources; use
+     * <code>containerOverrides</code> instead.</p> 
      */
     inline bool NodeOverridesHasBeenSet() const { return m_nodeOverridesHasBeenSet; }
 
     /**
      * <p>A list of node overrides in JSON format that specify the node range to target
-     * and the container overrides for that node range.</p>
+     * and the container overrides for that node range.</p>  <p>This parameter
+     * isn't applicable to jobs running on Fargate resources; use
+     * <code>containerOverrides</code> instead.</p> 
      */
     inline void SetNodeOverrides(const NodeOverrides& value) { m_nodeOverridesHasBeenSet = true; m_nodeOverrides = value; }
 
     /**
      * <p>A list of node overrides in JSON format that specify the node range to target
-     * and the container overrides for that node range.</p>
+     * and the container overrides for that node range.</p>  <p>This parameter
+     * isn't applicable to jobs running on Fargate resources; use
+     * <code>containerOverrides</code> instead.</p> 
      */
     inline void SetNodeOverrides(NodeOverrides&& value) { m_nodeOverridesHasBeenSet = true; m_nodeOverrides = std::move(value); }
 
     /**
      * <p>A list of node overrides in JSON format that specify the node range to target
-     * and the container overrides for that node range.</p>
+     * and the container overrides for that node range.</p>  <p>This parameter
+     * isn't applicable to jobs running on Fargate resources; use
+     * <code>containerOverrides</code> instead.</p> 
      */
     inline SubmitJobRequest& WithNodeOverrides(const NodeOverrides& value) { SetNodeOverrides(value); return *this;}
 
     /**
      * <p>A list of node overrides in JSON format that specify the node range to target
-     * and the container overrides for that node range.</p>
+     * and the container overrides for that node range.</p>  <p>This parameter
+     * isn't applicable to jobs running on Fargate resources; use
+     * <code>containerOverrides</code> instead.</p> 
      */
     inline SubmitJobRequest& WithNodeOverrides(NodeOverrides&& value) { SetNodeOverrides(std::move(value)); return *this;}
 
@@ -609,12 +617,57 @@ namespace Model
     inline SubmitJobRequest& WithRetryStrategy(RetryStrategy&& value) { SetRetryStrategy(std::move(value)); return *this;}
 
 
+    /**
+     * <p>Specifies whether to propagate the tags from the job or job definition to the
+     * corresponding Amazon ECS task. If no value is specified, the tags aren't
+     * propagated. Tags can only be propagated to the tasks during task creation. For
+     * tags with the same name, job tags are given priority over job definitions tags.
+     * If the total number of combined tags from the job and job definition is over 50,
+     * the job is moved to the <code>FAILED</code> state. When specified, this
+     * overrides the tag propagation setting in the job definition.</p>
+     */
+    inline bool GetPropagateTags() const{ return m_propagateTags; }
+
+    /**
+     * <p>Specifies whether to propagate the tags from the job or job definition to the
+     * corresponding Amazon ECS task. If no value is specified, the tags aren't
+     * propagated. Tags can only be propagated to the tasks during task creation. For
+     * tags with the same name, job tags are given priority over job definitions tags.
+     * If the total number of combined tags from the job and job definition is over 50,
+     * the job is moved to the <code>FAILED</code> state. When specified, this
+     * overrides the tag propagation setting in the job definition.</p>
+     */
+    inline bool PropagateTagsHasBeenSet() const { return m_propagateTagsHasBeenSet; }
+
+    /**
+     * <p>Specifies whether to propagate the tags from the job or job definition to the
+     * corresponding Amazon ECS task. If no value is specified, the tags aren't
+     * propagated. Tags can only be propagated to the tasks during task creation. For
+     * tags with the same name, job tags are given priority over job definitions tags.
+     * If the total number of combined tags from the job and job definition is over 50,
+     * the job is moved to the <code>FAILED</code> state. When specified, this
+     * overrides the tag propagation setting in the job definition.</p>
+     */
+    inline void SetPropagateTags(bool value) { m_propagateTagsHasBeenSet = true; m_propagateTags = value; }
+
+    /**
+     * <p>Specifies whether to propagate the tags from the job or job definition to the
+     * corresponding Amazon ECS task. If no value is specified, the tags aren't
+     * propagated. Tags can only be propagated to the tasks during task creation. For
+     * tags with the same name, job tags are given priority over job definitions tags.
+     * If the total number of combined tags from the job and job definition is over 50,
+     * the job is moved to the <code>FAILED</code> state. When specified, this
+     * overrides the tag propagation setting in the job definition.</p>
+     */
+    inline SubmitJobRequest& WithPropagateTags(bool value) { SetPropagateTags(value); return *this;}
+
+
     /**
      * <p>The timeout configuration for this <a>SubmitJob</a> operation. You can
      * specify a timeout duration after which AWS Batch terminates your jobs if they
-     * have not finished. If a job is terminated due to a timeout, it is not retried.
-     * The minimum value for the timeout is 60 seconds. This configuration overrides
-     * any timeout configuration specified in the job definition. For array jobs, child
+     * haven't finished. If a job is terminated due to a timeout, it isn't retried. The
+     * minimum value for the timeout is 60 seconds. This configuration overrides any
+     * timeout configuration specified in the job definition. For array jobs, child
      * jobs have the same timeout configuration as the parent job. For more
      * information, see <a
      * href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/job_timeouts.html">Job
@@ -625,9 +678,9 @@ namespace Model
     /**
      * <p>The timeout configuration for this <a>SubmitJob</a> operation. You can
      * specify a timeout duration after which AWS Batch terminates your jobs if they
-     * have not finished. If a job is terminated due to a timeout, it is not retried.
-     * The minimum value for the timeout is 60 seconds. This configuration overrides
-     * any timeout configuration specified in the job definition. For array jobs, child
+     * haven't finished. If a job is terminated due to a timeout, it isn't retried. The
+     * minimum value for the timeout is 60 seconds. This configuration overrides any
+     * timeout configuration specified in the job definition. For array jobs, child
      * jobs have the same timeout configuration as the parent job. For more
      * information, see <a
      * href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/job_timeouts.html">Job
@@ -638,9 +691,9 @@ namespace Model
     /**
      * <p>The timeout configuration for this <a>SubmitJob</a> operation. You can
      * specify a timeout duration after which AWS Batch terminates your jobs if they
-     * have not finished. If a job is terminated due to a timeout, it is not retried.
-     * The minimum value for the timeout is 60 seconds. This configuration overrides
-     * any timeout configuration specified in the job definition. For array jobs, child
+     * haven't finished. If a job is terminated due to a timeout, it isn't retried. The
+     * minimum value for the timeout is 60 seconds. This configuration overrides any
+     * timeout configuration specified in the job definition. For array jobs, child
      * jobs have the same timeout configuration as the parent job. For more
      * information, see <a
      * href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/job_timeouts.html">Job
@@ -651,9 +704,9 @@ namespace Model
     /**
      * <p>The timeout configuration for this <a>SubmitJob</a> operation. You can
      * specify a timeout duration after which AWS Batch terminates your jobs if they
-     * have not finished. If a job is terminated due to a timeout, it is not retried.
-     * The minimum value for the timeout is 60 seconds. This configuration overrides
-     * any timeout configuration specified in the job definition. For array jobs, child
+     * haven't finished. If a job is terminated due to a timeout, it isn't retried. The
+     * minimum value for the timeout is 60 seconds. This configuration overrides any
+     * timeout configuration specified in the job definition. For array jobs, child
      * jobs have the same timeout configuration as the parent job. For more
      * information, see <a
      * href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/job_timeouts.html">Job
@@ -664,9 +717,9 @@ namespace Model
     /**
      * <p>The timeout configuration for this <a>SubmitJob</a> operation. You can
      * specify a timeout duration after which AWS Batch terminates your jobs if they
-     * have not finished. If a job is terminated due to a timeout, it is not retried.
-     * The minimum value for the timeout is 60 seconds. This configuration overrides
-     * any timeout configuration specified in the job definition. For array jobs, child
+     * haven't finished. If a job is terminated due to a timeout, it isn't retried. The
+     * minimum value for the timeout is 60 seconds. This configuration overrides any
+     * timeout configuration specified in the job definition. For array jobs, child
      * jobs have the same timeout configuration as the parent job. For more
      * information, see <a
      * href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/job_timeouts.html">Job
@@ -677,9 +730,9 @@ namespace Model
     /**
      * <p>The timeout configuration for this <a>SubmitJob</a> operation. You can
      * specify a timeout duration after which AWS Batch terminates your jobs if they
-     * have not finished. If a job is terminated due to a timeout, it is not retried.
-     * The minimum value for the timeout is 60 seconds. This configuration overrides
-     * any timeout configuration specified in the job definition. For array jobs, child
+     * haven't finished. If a job is terminated due to a timeout, it isn't retried. The
+     * minimum value for the timeout is 60 seconds. This configuration overrides any
+     * timeout configuration specified in the job definition. For array jobs, child
      * jobs have the same timeout configuration as the parent job. For more
      * information, see <a
      * href="https://docs.aws.amazon.com/AmazonECS/latest/developerguide/job_timeouts.html">Job
@@ -687,6 +740,124 @@ namespace Model
      */
     inline SubmitJobRequest& WithTimeout(JobTimeout&& value) { SetTimeout(std::move(value)); return *this;}
 
+
+    /**
+     * <p>The tags that you apply to the job request to help you categorize and
+     * organize your resources. Each tag consists of a key and an optional value. For
+     * more information, see <a
+     * href="https://docs.aws.amazon.com/general/latest/gr/aws_tagging.html">Tagging
+     * AWS Resources</a> in <i>AWS General Reference</i>.</p>
+     */
+    inline const Aws::Map<Aws::String, Aws::String>& GetTags() const{ return m_tags; }
+
+    /**
+     * <p>The tags that you apply to the job request to help you categorize and
+     * organize your resources. Each tag consists of a key and an optional value. For
+     * more information, see <a
+     * href="https://docs.aws.amazon.com/general/latest/gr/aws_tagging.html">Tagging
+     * AWS Resources</a> in <i>AWS General Reference</i>.</p>
+     */
+    inline bool TagsHasBeenSet() const { return m_tagsHasBeenSet; }
+
+    /**
+     * <p>The tags that you apply to the job request to help you categorize and
+     * organize your resources. Each tag consists of a key and an optional value. For
+     * more information, see <a
+     * href="https://docs.aws.amazon.com/general/latest/gr/aws_tagging.html">Tagging
+     * AWS Resources</a> in <i>AWS General Reference</i>.</p>
+     */
+    inline void SetTags(const Aws::Map<Aws::String, Aws::String>& value) { m_tagsHasBeenSet = true; m_tags = value; }
+
+    /**
+     * <p>The tags that you apply to the job request to help you categorize and
+     * organize your resources. Each tag consists of a key and an optional value. For
+     * more information, see <a
+     * href="https://docs.aws.amazon.com/general/latest/gr/aws_tagging.html">Tagging
+     * AWS Resources</a> in <i>AWS General Reference</i>.</p>
+     */
+    inline void SetTags(Aws::Map<Aws::String, Aws::String>&& value) { m_tagsHasBeenSet = true; m_tags = std::move(value); }
+
+    /**
+     * <p>The tags that you apply to the job request to help you categorize and
+     * organize your resources. Each tag consists of a key and an optional value. For
+     * more information, see <a
+     * href="https://docs.aws.amazon.com/general/latest/gr/aws_tagging.html">Tagging
+     * AWS Resources</a> in <i>AWS General Reference</i>.</p>
+     */
+    inline SubmitJobRequest& WithTags(const Aws::Map<Aws::String, Aws::String>& value) { SetTags(value); return *this;}
+
+    /**
+     * <p>The tags that you apply to the job request to help you categorize and
+     * organize your resources. Each tag consists of a key and an optional value. For
+     * more information, see <a
+     * href="https://docs.aws.amazon.com/general/latest/gr/aws_tagging.html">Tagging
+     * AWS Resources</a> in <i>AWS General Reference</i>.</p>
+     */
+    inline SubmitJobRequest& WithTags(Aws::Map<Aws::String, Aws::String>&& value) { SetTags(std::move(value)); return *this;}
+
+    /**
+     * <p>The tags that you apply to the job request to help you categorize and
+     * organize your resources. Each tag consists of a key and an optional value. For
+     * more information, see <a
+     * href="https://docs.aws.amazon.com/general/latest/gr/aws_tagging.html">Tagging
+     * AWS Resources</a> in <i>AWS General Reference</i>.</p>
+     */
+    inline SubmitJobRequest& AddTags(const Aws::String& key, const Aws::String& value) { m_tagsHasBeenSet = true; m_tags.emplace(key, value); return *this; }
+
+    /**
+     * <p>The tags that you apply to the job request to help you categorize and
+     * organize your resources. Each tag consists of a key and an optional value. For
+     * more information, see <a
+     * href="https://docs.aws.amazon.com/general/latest/gr/aws_tagging.html">Tagging
+     * AWS Resources</a> in <i>AWS General Reference</i>.</p>
+     */
+    inline SubmitJobRequest& AddTags(Aws::String&& key, const Aws::String& value) { m_tagsHasBeenSet = true; m_tags.emplace(std::move(key), value); return *this; }
+
+    /**
+     * <p>The tags that you apply to the job request to help you categorize and
+     * organize your resources. Each tag consists of a key and an optional value. For
+     * more information, see <a
+     * href="https://docs.aws.amazon.com/general/latest/gr/aws_tagging.html">Tagging
+     * AWS Resources</a> in <i>AWS General Reference</i>.</p>
+     */
+    inline SubmitJobRequest& AddTags(const Aws::String& key, Aws::String&& value) { m_tagsHasBeenSet = true; m_tags.emplace(key, std::move(value)); return *this; }
+
+    /**
+     * <p>The tags that you apply to the job request to help you categorize and
+     * organize your resources. Each tag consists of a key and an optional value. For
+     * more information, see <a
+     * href="https://docs.aws.amazon.com/general/latest/gr/aws_tagging.html">Tagging
+     * AWS Resources</a> in <i>AWS General Reference</i>.</p>
+     */
+    inline SubmitJobRequest& AddTags(Aws::String&& key, Aws::String&& value) { m_tagsHasBeenSet = true; m_tags.emplace(std::move(key), std::move(value)); return *this; }
+
+    /**
+     * <p>The tags that you apply to the job request to help you categorize and
+     * organize your resources. Each tag consists of a key and an optional value. For
+     * more information, see <a
+     * href="https://docs.aws.amazon.com/general/latest/gr/aws_tagging.html">Tagging
+     * AWS Resources</a> in <i>AWS General Reference</i>.</p>
+     */
+    inline SubmitJobRequest& AddTags(const char* key, Aws::String&& value) { m_tagsHasBeenSet = true; m_tags.emplace(key, std::move(value)); return *this; }
+
+    /**
+     * <p>The tags that you apply to the job request to help you categorize and
+     * organize your resources. Each tag consists of a key and an optional value. For
+     * more information, see <a
+     * href="https://docs.aws.amazon.com/general/latest/gr/aws_tagging.html">Tagging
+     * AWS Resources</a> in <i>AWS General Reference</i>.</p>
+     */
+    inline SubmitJobRequest& AddTags(Aws::String&& key, const char* value) { m_tagsHasBeenSet = true; m_tags.emplace(std::move(key), value); return *this; }
+
+    /**
+     * <p>The tags that you apply to the job request to help you categorize and
+     * organize your resources. Each tag consists of a key and an optional value. For
+     * more information, see <a
+     * href="https://docs.aws.amazon.com/general/latest/gr/aws_tagging.html">Tagging
+     * AWS Resources</a> in <i>AWS General Reference</i>.</p>
+     */
+    inline SubmitJobRequest& AddTags(const char* key, const char* value) { m_tagsHasBeenSet = true; m_tags.emplace(key, value); return *this; }
+
   private:
 
     Aws::String m_jobName;
@@ -716,8 +887,14 @@ namespace Model
     RetryStrategy m_retryStrategy;
     bool m_retryStrategyHasBeenSet;
 
+    bool m_propagateTags;
+    bool m_propagateTagsHasBeenSet;
+
     JobTimeout m_timeout;
     bool m_timeoutHasBeenSet;
+
+    Aws::Map<Aws::String, Aws::String> m_tags;
+    bool m_tagsHasBeenSet;
   };
 
 } // namespace Model
