@@ -16,6 +16,10 @@ GetTagsRequest::GetTagsRequest() :
     m_searchStringHasBeenSet(false),
     m_timePeriodHasBeenSet(false),
     m_tagKeyHasBeenSet(false),
+    m_filterHasBeenSet(false),
+    m_sortByHasBeenSet(false),
+    m_maxResults(0),
+    m_maxResultsHasBeenSet(false),
     m_nextPageTokenHasBeenSet(false)
 {
 }
@@ -42,6 +46,29 @@ Aws::String GetTagsRequest::SerializePayload() const
 
   }
 
+  if(m_filterHasBeenSet)
+  {
+   payload.WithObject("Filter", m_filter.Jsonize());
+
+  }
+
+  if(m_sortByHasBeenSet)
+  {
+   Array<JsonValue> sortByJsonList(m_sortBy.size());
+   for(unsigned sortByIndex = 0; sortByIndex < sortByJsonList.GetLength(); ++sortByIndex)
+   {
+     sortByJsonList[sortByIndex].AsObject(m_sortBy[sortByIndex].Jsonize());
+   }
+   payload.WithArray("SortBy", std::move(sortByJsonList));
+
+  }
+
+  if(m_maxResultsHasBeenSet)
+  {
+   payload.WithInteger("MaxResults", m_maxResults);
+
+  }
+
   if(m_nextPageTokenHasBeenSet)
   {
    payload.WithString("NextPageToken", m_nextPageToken);
