@@ -1,17 +1,7 @@
-﻿/*
-* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
-*
-* Licensed under the Apache License, Version 2.0 (the "License").
-* You may not use this file except in compliance with the License.
-* A copy of the License is located at
-*
-*  http://aws.amazon.com/apache2.0
-*
-* or in the "license" file accompanying this file. This file is distributed
-* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
-* express or implied. See the License for the specific language governing
-* permissions and limitations under the License.
-*/
+﻿/**
+ * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
+ * SPDX-License-Identifier: Apache-2.0.
+ */
 
 #pragma once
 #include <aws/ec2/EC2_EXPORTS.h>
@@ -250,32 +240,32 @@ namespace Model
 
 
     /**
-     * <p>The type of interface.</p>
+     * <p>The type of network interface.</p>
      */
     inline const NetworkInterfaceType& GetInterfaceType() const{ return m_interfaceType; }
 
     /**
-     * <p>The type of interface.</p>
+     * <p>The type of network interface.</p>
      */
     inline bool InterfaceTypeHasBeenSet() const { return m_interfaceTypeHasBeenSet; }
 
     /**
-     * <p>The type of interface.</p>
+     * <p>The type of network interface.</p>
      */
     inline void SetInterfaceType(const NetworkInterfaceType& value) { m_interfaceTypeHasBeenSet = true; m_interfaceType = value; }
 
     /**
-     * <p>The type of interface.</p>
+     * <p>The type of network interface.</p>
      */
     inline void SetInterfaceType(NetworkInterfaceType&& value) { m_interfaceTypeHasBeenSet = true; m_interfaceType = std::move(value); }
 
     /**
-     * <p>The type of interface.</p>
+     * <p>The type of network interface.</p>
      */
     inline NetworkInterface& WithInterfaceType(const NetworkInterfaceType& value) { SetInterfaceType(value); return *this;}
 
     /**
-     * <p>The type of interface.</p>
+     * <p>The type of network interface.</p>
      */
     inline NetworkInterface& WithInterfaceType(NetworkInterfaceType&& value) { SetInterfaceType(std::move(value)); return *this;}
 
@@ -403,6 +393,47 @@ namespace Model
     inline NetworkInterface& WithNetworkInterfaceId(const char* value) { SetNetworkInterfaceId(value); return *this;}
 
 
+    /**
+     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
+     */
+    inline const Aws::String& GetOutpostArn() const{ return m_outpostArn; }
+
+    /**
+     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
+     */
+    inline bool OutpostArnHasBeenSet() const { return m_outpostArnHasBeenSet; }
+
+    /**
+     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
+     */
+    inline void SetOutpostArn(const Aws::String& value) { m_outpostArnHasBeenSet = true; m_outpostArn = value; }
+
+    /**
+     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
+     */
+    inline void SetOutpostArn(Aws::String&& value) { m_outpostArnHasBeenSet = true; m_outpostArn = std::move(value); }
+
+    /**
+     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
+     */
+    inline void SetOutpostArn(const char* value) { m_outpostArnHasBeenSet = true; m_outpostArn.assign(value); }
+
+    /**
+     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
+     */
+    inline NetworkInterface& WithOutpostArn(const Aws::String& value) { SetOutpostArn(value); return *this;}
+
+    /**
+     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
+     */
+    inline NetworkInterface& WithOutpostArn(Aws::String&& value) { SetOutpostArn(std::move(value)); return *this;}
+
+    /**
+     * <p>The Amazon Resource Name (ARN) of the Outpost.</p>
+     */
+    inline NetworkInterface& WithOutpostArn(const char* value) { SetOutpostArn(value); return *this;}
+
+
     /**
      * <p>The AWS account ID of the owner of the network interface.</p>
      */
@@ -568,50 +599,50 @@ namespace Model
 
 
     /**
-     * <p>The ID of the entity that launched the instance on your behalf (for example,
-     * AWS Management Console or Auto Scaling).</p>
+     * <p>The alias or AWS account ID of the principal or service that created the
+     * network interface.</p>
      */
     inline const Aws::String& GetRequesterId() const{ return m_requesterId; }
 
     /**
-     * <p>The ID of the entity that launched the instance on your behalf (for example,
-     * AWS Management Console or Auto Scaling).</p>
+     * <p>The alias or AWS account ID of the principal or service that created the
+     * network interface.</p>
      */
     inline bool RequesterIdHasBeenSet() const { return m_requesterIdHasBeenSet; }
 
     /**
-     * <p>The ID of the entity that launched the instance on your behalf (for example,
-     * AWS Management Console or Auto Scaling).</p>
+     * <p>The alias or AWS account ID of the principal or service that created the
+     * network interface.</p>
      */
     inline void SetRequesterId(const Aws::String& value) { m_requesterIdHasBeenSet = true; m_requesterId = value; }
 
     /**
-     * <p>The ID of the entity that launched the instance on your behalf (for example,
-     * AWS Management Console or Auto Scaling).</p>
+     * <p>The alias or AWS account ID of the principal or service that created the
+     * network interface.</p>
      */
     inline void SetRequesterId(Aws::String&& value) { m_requesterIdHasBeenSet = true; m_requesterId = std::move(value); }
 
     /**
-     * <p>The ID of the entity that launched the instance on your behalf (for example,
-     * AWS Management Console or Auto Scaling).</p>
+     * <p>The alias or AWS account ID of the principal or service that created the
+     * network interface.</p>
      */
     inline void SetRequesterId(const char* value) { m_requesterIdHasBeenSet = true; m_requesterId.assign(value); }
 
     /**
-     * <p>The ID of the entity that launched the instance on your behalf (for example,
-     * AWS Management Console or Auto Scaling).</p>
+     * <p>The alias or AWS account ID of the principal or service that created the
+     * network interface.</p>
      */
     inline NetworkInterface& WithRequesterId(const Aws::String& value) { SetRequesterId(value); return *this;}
 
     /**
-     * <p>The ID of the entity that launched the instance on your behalf (for example,
-     * AWS Management Console or Auto Scaling).</p>
+     * <p>The alias or AWS account ID of the principal or service that created the
+     * network interface.</p>
      */
     inline NetworkInterface& WithRequesterId(Aws::String&& value) { SetRequesterId(std::move(value)); return *this;}
 
     /**
-     * <p>The ID of the entity that launched the instance on your behalf (for example,
-     * AWS Management Console or Auto Scaling).</p>
+     * <p>The alias or AWS account ID of the principal or service that created the
+     * network interface.</p>
      */
     inline NetworkInterface& WithRequesterId(const char* value) { SetRequesterId(value); return *this;}
 
@@ -638,22 +669,22 @@ namespace Model
 
 
     /**
-     * <p>Indicates whether traffic to or from the instance is validated.</p>
+     * <p>Indicates whether source/destination checking is enabled.</p>
      */
     inline bool GetSourceDestCheck() const{ return m_sourceDestCheck; }
 
     /**
-     * <p>Indicates whether traffic to or from the instance is validated.</p>
+     * <p>Indicates whether source/destination checking is enabled.</p>
      */
     inline bool SourceDestCheckHasBeenSet() const { return m_sourceDestCheckHasBeenSet; }
 
     /**
-     * <p>Indicates whether traffic to or from the instance is validated.</p>
+     * <p>Indicates whether source/destination checking is enabled.</p>
      */
     inline void SetSourceDestCheck(bool value) { m_sourceDestCheckHasBeenSet = true; m_sourceDestCheck = value; }
 
     /**
-     * <p>Indicates whether traffic to or from the instance is validated.</p>
+     * <p>Indicates whether source/destination checking is enabled.</p>
      */
     inline NetworkInterface& WithSourceDestCheck(bool value) { SetSourceDestCheck(value); return *this;}
 
@@ -840,6 +871,9 @@ namespace Model
     Aws::String m_networkInterfaceId;
     bool m_networkInterfaceIdHasBeenSet;
 
+    Aws::String m_outpostArn;
+    bool m_outpostArnHasBeenSet;
+
     Aws::String m_ownerId;
     bool m_ownerIdHasBeenSet;
 
