@@ -1,17 +1,7 @@
-﻿/*
-* Copyright 2010-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
-*
-* Licensed under the Apache License, Version 2.0 (the "License").
-* You may not use this file except in compliance with the License.
-* A copy of the License is located at
-*
-*  http://aws.amazon.com/apache2.0
-*
-* or in the "license" file accompanying this file. This file is distributed
-* on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
-* express or implied. See the License for the specific language governing
-* permissions and limitations under the License.
-*/
+﻿/**
+ * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
+ * SPDX-License-Identifier: Apache-2.0.
+ */
 
 #include <aws/ec2/model/AssociateSubnetCidrBlockRequest.h>
 #include <aws/core/utils/StringUtils.h>
@@ -21,8 +11,8 @@ using namespace Aws::EC2::Model;
 using namespace Aws::Utils;
 
 AssociateSubnetCidrBlockRequest::AssociateSubnetCidrBlockRequest() : 
-    m_ipv6CidrBlockHasBeenSet(false),
-    m_subnetIdHasBeenSet(false)
+    m_subnetIdHasBeenSet(false),
+    m_ipv6CidrBlockHasBeenSet(false)
 {
 }
 
@@ -30,16 +20,16 @@ Aws::String AssociateSubnetCidrBlockRequest::SerializePayload() const
 {
   Aws::StringStream ss;
   ss << "Action=AssociateSubnetCidrBlock&";
-  if(m_ipv6CidrBlockHasBeenSet)
-  {
-    ss << "Ipv6CidrBlock=" << StringUtils::URLEncode(m_ipv6CidrBlock.c_str()) << "&";
-  }
-
   if(m_subnetIdHasBeenSet)
   {
     ss << "SubnetId=" << StringUtils::URLEncode(m_subnetId.c_str()) << "&";
   }
 
+  if(m_ipv6CidrBlockHasBeenSet)
+  {
+    ss << "Ipv6CidrBlock=" << StringUtils::URLEncode(m_ipv6CidrBlock.c_str()) << "&";
+  }
+
   ss << "Version=2016-11-15";
   return ss.str();
 }
