@@ -388,7 +388,6 @@ testRolesInfoCommand(rst.getPrimary());
 rst.stopSet();
 
 print("Testing sharded cluster");
-// TODO: Remove 'shardAsReplicaSet: false' when SERVER-32672 is fixed.
 var st = new ShardingTest({
     mongos: 2,
     config: 3,
@@ -397,8 +396,7 @@ var st = new ShardingTest({
     other: {
         mongosOptions: {bind_ip_all: "", auth: null},
         configOptions: {auth: null},
-        shardOptions: {auth: null},
-        shardAsReplicaSet: false
+        shardOptions: {auth: null}
     }
 });
 testRestrictionCreationAndEnforcement(
