@@ -30,13 +30,13 @@
 #include "mongo/db/index/btree_key_generator.h"
 
 #include <boost/optional.hpp>
+#include <memory>
 
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/bson/dotted_path_support.h"
 #include "mongo/db/field_ref.h"
 #include "mongo/db/query/collation/collation_index_key.h"
 #include "mongo/db/query/collation/collator_interface.h"
-#include "mongo/stdx/memory.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/str.h"
 
@@ -53,28 +53,63 @@ const BSONElement nullElt = nullObj.firstElement();
 const BSONObj undefinedObj = BSON("" << BSONUndefined);
 const BSONElement undefinedElt = undefinedObj.firstElement();
 
+/**
+ * Returns the non-array element at the specified path. This function returns an empty BSON element
+ * if the path doesn't exist.
+ *
+ * The 'path' can be specified using a dotted notation in order to traverse through embedded
+ * objects.
+ *
+ * This function must only be used when there is no an array element along the 'path'. The caller is
+ * responsible to ensure this invariant holds.
+ */
+BSONElement extractNonArrayElementAtPath(const BSONObj& obj, StringData path) {
+    static const auto kEmptyElt = BSONElement{};
+
+    auto&& [elt, tail] = [&]() -> std::pair<BSONElement, StringData> {
+        if (auto dotOffset = path.find("."); dotOffset != std::string::npos) {
+            return {obj.getField(path.substr(0, dotOffset)), path.substr(dotOffset + 1)};
+        }
+        return {obj.getField(path), ""_sd};
+    }();
+    invariant(elt.type() != BSONType::Array);
+
+    if (elt.eoo()) {
+        return kEmptyElt;
+    } else if (tail.empty()) {
+        return elt;
+    } else if (elt.type() == BSONType::Object) {
+        return extractNonArrayElementAtPath(elt.embeddedObject(), tail);
+    }
+    // We found a scalar element, but there is more path to traverse, e.g. {a: 1} with a path of
+    // "a.b".
+    return kEmptyElt;
+}
 }  // namespace
 
 BtreeKeyGenerator::BtreeKeyGenerator(std::vector<const char*> fieldNames,
                                      std::vector<BSONElement> fixed,
                                      bool isSparse,
-                                     const CollatorInterface* collator)
-    : _fieldNames(fieldNames),
+                                     const CollatorInterface* collator,
+                                     KeyString::Version keyStringVersion,
+                                     Ordering ordering)
+    : _keyStringVersion(keyStringVersion),
+      _ordering(ordering),
+      _fieldNames(fieldNames),
+      _isIdIndex(fieldNames.size() == 1 && std::string("_id") == fieldNames[0]),
       _isSparse(isSparse),
+      _nullKeyString(_buildNullKeyString()),
       _fixed(fixed),
       _emptyPositionalInfo(fieldNames.size()),
       _collator(collator) {
-    BSONObjBuilder nullKeyBuilder;
-    for (size_t i = 0; i < fieldNames.size(); ++i) {
-        nullKeyBuilder.appendNull("");
-    }
-    _nullKey = nullKeyBuilder.obj();
 
-    _isIdIndex = fieldNames.size() == 1 && std::string("_id") == fieldNames[0];
     for (const char* fieldName : fieldNames) {
-        size_t pathLength = FieldRef{fieldName}.numParts();
+        FieldRef fieldRef{fieldName};
+        auto pathLength = fieldRef.numParts();
         invariant(pathLength > 0);
         _pathLengths.push_back(pathLength);
+        _pathsContainPositionalComponent =
+            _pathsContainPositionalComponent || fieldRef.hasNumericPathComponents();
     }
 }
 
@@ -130,13 +165,14 @@ BSONElement BtreeKeyGenerator::_extractNextElement(const BSONObj& obj,
 void BtreeKeyGenerator::_getKeysArrEltFixed(std::vector<const char*>* fieldNames,
                                             std::vector<BSONElement>* fixed,
                                             const BSONElement& arrEntry,
-                                            BSONObjSet* keys,
+                                            KeyStringSet* keys,
                                             unsigned numNotFound,
                                             const BSONElement& arrObjElt,
                                             const std::set<size_t>& arrIdxs,
                                             bool mayExpandArrayUnembedded,
                                             const std::vector<PositionalPathInfo>& positionalInfo,
-                                            MultikeyPaths* multikeyPaths) const {
+                                            MultikeyPaths* multikeyPaths,
+                                            boost::optional<RecordId> id) const {
     // Set up any terminal array values.
     for (const auto idx : arrIdxs) {
         if (*(*fieldNames)[idx] == '\0') {
@@ -151,29 +187,38 @@ void BtreeKeyGenerator::_getKeysArrEltFixed(std::vector<const char*>* fieldNames
                       keys,
                       numNotFound,
                       positionalInfo,
-                      multikeyPaths);
+                      multikeyPaths,
+                      id);
 }
 
 void BtreeKeyGenerator::getKeys(const BSONObj& obj,
-                                BSONObjSet* keys,
-                                MultikeyPaths* multikeyPaths) const {
+                                bool skipMultikey,
+                                KeyStringSet* keys,
+                                MultikeyPaths* multikeyPaths,
+                                boost::optional<RecordId> id) const {
     if (_isIdIndex) {
         // we special case for speed
         BSONElement e = obj["_id"];
         if (e.eoo()) {
-            keys->insert(_nullKey);
-        } else if (_collator) {
-            BSONObjBuilder b;
-            CollationIndexKey::collationAwareIndexKeyAppend(e, _collator, &b);
-
-            // Insert a copy so its buffer size fits the object size.
-            keys->insert(b.obj().copy());
+            keys->insert(_nullKeyString);
         } else {
-            int size = e.size() + 5 /* bson over head*/ - 3 /* remove _id string */;
-            BSONObjBuilder b(size);
-            b.appendAs(e, "");
-            keys->insert(b.obj());
-            invariant(keys->begin()->objsize() == size);
+            KeyString::Builder keyString(_keyStringVersion, _ordering);
+
+            if (_collator) {
+                keyString.appendBSONElement(e, [&](StringData stringData) {
+                    return _collator->getComparisonString(stringData);
+                });
+            } else {
+                keyString.appendBSONElement(e);
+            }
+
+            if (id) {
+                keyString.appendRecordId(*id);
+            }
+            /*
+             * Insert a copy so its buffer size fits the key size.
+             */
+            keys->insert(keyString.getValueCopy());
         }
 
         // The {_id: 1} index can never be multikey because the _id field isn't allowed to be an
@@ -181,6 +226,14 @@ void BtreeKeyGenerator::getKeys(const BSONObj& obj,
         if (multikeyPaths) {
             multikeyPaths->resize(1);
         }
+    } else if (skipMultikey && !_pathsContainPositionalComponent) {
+        // This index doesn't contain array values. We therefore always set 'multikeyPaths' as
+        // [[ ], [], ...].
+        if (multikeyPaths) {
+            invariant(multikeyPaths->empty());
+            multikeyPaths->resize(_fieldNames.size());
+        }
+        _getKeysWithoutArray(obj, id, keys);
     } else {
         if (multikeyPaths) {
             invariant(multikeyPaths->empty());
@@ -188,20 +241,55 @@ void BtreeKeyGenerator::getKeys(const BSONObj& obj,
         }
         // '_fieldNames' and '_fixed' are passed by value so that their copies can be mutated as
         // part of the _getKeysWithArray method.
-        _getKeysWithArray(_fieldNames, _fixed, obj, keys, 0, _emptyPositionalInfo, multikeyPaths);
+        _getKeysWithArray(
+            _fieldNames, _fixed, obj, keys, 0, _emptyPositionalInfo, multikeyPaths, id);
     }
+
     if (keys->empty() && !_isSparse) {
-        keys->insert(_nullKey);
+        keys->insert(_nullKeyString);
     }
 }
 
+void BtreeKeyGenerator::_getKeysWithoutArray(const BSONObj& obj,
+                                             boost::optional<RecordId> id,
+                                             KeyStringSet* keys) const {
+
+    KeyString::HeapBuilder keyString{_keyStringVersion, _ordering};
+    size_t numNotFound{0};
+
+    for (auto&& fieldName : _fieldNames) {
+        auto elem = extractNonArrayElementAtPath(obj, fieldName);
+        if (elem.eoo()) {
+            ++numNotFound;
+        }
+
+        if (_collator) {
+            keyString.appendBSONElement(elem, [&](StringData stringData) {
+                return _collator->getComparisonString(stringData);
+            });
+        } else {
+            keyString.appendBSONElement(elem);
+        }
+    }
+
+    if (_isSparse && numNotFound == _fieldNames.size()) {
+        return;
+    }
+
+    if (id) {
+        keyString.appendRecordId(*id);
+    }
+    keys->insert(keyString.release());
+}
+
 void BtreeKeyGenerator::_getKeysWithArray(std::vector<const char*> fieldNames,
                                           std::vector<BSONElement> fixed,
                                           const BSONObj& obj,
-                                          BSONObjSet* keys,
+                                          KeyStringSet* keys,
                                           unsigned numNotFound,
                                           const std::vector<PositionalPathInfo>& positionalInfo,
-                                          MultikeyPaths* multikeyPaths) const {
+                                          MultikeyPaths* multikeyPaths,
+                                          boost::optional<RecordId> id) const {
     BSONElement arrElt;
 
     // A set containing the position of any indexed fields in the key pattern that traverse through
@@ -270,11 +358,20 @@ void BtreeKeyGenerator::_getKeysWithArray(std::vector<const char*> fieldNames,
         if (_isSparse && numNotFound == fieldNames.size()) {
             return;
         }
-        BSONObjBuilder b(_sizeTracker);
-        for (std::vector<BSONElement>::iterator i = fixed.begin(); i != fixed.end(); ++i) {
-            CollationIndexKey::collationAwareIndexKeyAppend(*i, _collator, &b);
+        KeyString::HeapBuilder keyString(_keyStringVersion, _ordering);
+        for (const auto& elem : fixed) {
+            if (_collator) {
+                keyString.appendBSONElement(elem, [&](StringData stringData) {
+                    return _collator->getComparisonString(stringData);
+                });
+            } else {
+                keyString.appendBSONElement(elem);
+            }
         }
-        keys->insert(b.obj());
+        if (id) {
+            keyString.appendRecordId(*id);
+        }
+        keys->insert(keyString.release());
     } else if (arrElt.embeddedObject().firstElement().eoo()) {
         // We've encountered an empty array.
         if (multikeyPaths && mayExpandArrayUnembedded) {
@@ -302,7 +399,8 @@ void BtreeKeyGenerator::_getKeysWithArray(std::vector<const char*> fieldNames,
                             arrIdxs,
                             true,
                             _emptyPositionalInfo,
-                            multikeyPaths);
+                            multikeyPaths,
+                            id);
     } else {
         BSONObj arrObj = arrElt.embeddedObject();
 
@@ -384,7 +482,8 @@ void BtreeKeyGenerator::_getKeysWithArray(std::vector<const char*> fieldNames,
                                 arrIdxs,
                                 mayExpandArrayUnembedded,
                                 subPositionalInfo,
-                                multikeyPaths);
+                                multikeyPaths,
+                                id);
         }
     }
 
@@ -398,4 +497,13 @@ void BtreeKeyGenerator::_getKeysWithArray(std::vector<const char*> fieldNames,
     }
 }
 
+KeyString::Value BtreeKeyGenerator::_buildNullKeyString() const {
+    BSONObjBuilder nullKeyBuilder;
+    for (size_t i = 0; i < _fieldNames.size(); ++i) {
+        nullKeyBuilder.appendNull("");
+    }
+    KeyString::HeapBuilder nullKeyString(_keyStringVersion, nullKeyBuilder.obj(), _ordering);
+    return nullKeyString.release();
+}
+
 }  // namespace mongo
