@@ -27,18 +27,18 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kSharding
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kSharding
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/s/collection_metadata.h"
 
+#include <memory>
+
 #include "mongo/bson/simple_bsonobj_comparator.h"
 #include "mongo/bson/util/builder.h"
 #include "mongo/db/bson/dotted_path_support.h"
 #include "mongo/s/catalog/type_chunk.h"
-#include "mongo/stdx/memory.h"
-#include "mongo/util/log.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
@@ -46,13 +46,13 @@ namespace mongo {
 CollectionMetadata::CollectionMetadata(std::shared_ptr<ChunkManager> cm, const ShardId& thisShardId)
     : _cm(std::move(cm)), _thisShardId(thisShardId) {}
 
-BSONObj CollectionMetadata::extractDocumentKey(const BSONObj& doc) const {
+BSONObj CollectionMetadata::extractDocumentKey(const ShardKeyPattern* shardKeyPattern,
+                                               const BSONObj& doc) {
     BSONObj key;
 
-    if (isSharded()) {
-        auto const& pattern = _cm->getShardKeyPattern();
-        key = dotted_path_support::extractElementsBasedOnTemplate(doc, pattern.toBSON());
-        if (pattern.hasId()) {
+    if (shardKeyPattern) {
+        key = dotted_path_support::extractElementsBasedOnTemplate(doc, shardKeyPattern->toBSON());
+        if (shardKeyPattern->hasId()) {
             return key;
         }
         // else, try to append an _id field from the document.
@@ -66,10 +66,14 @@ BSONObj CollectionMetadata::extractDocumentKey(const BSONObj& doc) const {
     return doc;
 }
 
+BSONObj CollectionMetadata::extractDocumentKey(const BSONObj& doc) const {
+    return extractDocumentKey(isSharded() ? &_cm->getShardKeyPattern() : nullptr, doc);
+}
+
 void CollectionMetadata::toBSONBasic(BSONObjBuilder& bb) const {
     if (isSharded()) {
         _cm->getVersion().appendLegacyWithField(&bb, "collVersion");
-        getShardVersion().appendLegacyWithField(&bb, "shardVersion");
+        getShardVersionForLogging().appendLegacyWithField(&bb, "shardVersion");
         bb.append("keyPattern", _cm->getShardKeyPattern().toBSON());
     } else {
         ChunkVersion::UNSHARDED().appendLegacyWithField(&bb, "collVersion");
@@ -80,23 +84,23 @@ void CollectionMetadata::toBSONBasic(BSONObjBuilder& bb) const {
 std::string CollectionMetadata::toStringBasic() const {
     if (isSharded()) {
         return str::stream() << "collection version: " << _cm->getVersion().toString()
-                             << ", shard version: " << getShardVersion().toString();
+                             << ", shard version: " << getShardVersionForLogging().toString();
     } else {
         return "collection version: <unsharded>";
     }
 }
 
-RangeMap CollectionMetadata::getChunks() const {
+RangeMap CollectionMetadata::getOwnedChunks() const {
     invariant(isSharded());
 
     RangeMap chunksMap(SimpleBSONObjComparator::kInstance.makeBSONObjIndexedMap<BSONObj>());
 
-    _cm->forEachChunk([this, &chunksMap](const auto& chunk) {
-        if (chunk.getShardId() == _thisShardId)
-            chunksMap.emplace_hint(chunksMap.end(), chunk.getMin(), chunk.getMax());
+    for (const auto& chunk : _cm->chunks()) {
+        if (chunk.getShardId() != _thisShardId)
+            continue;
 
-        return true;
-    });
+        chunksMap.emplace_hint(chunksMap.end(), chunk.getMin(), chunk.getMax());
+    }
 
     return chunksMap;
 }
@@ -104,13 +108,13 @@ RangeMap CollectionMetadata::getChunks() const {
 bool CollectionMetadata::getNextChunk(const BSONObj& lookupKey, ChunkType* chunk) const {
     invariant(isSharded());
 
-    auto nextChunk = _cm->getNextChunkOnShard(lookupKey, _thisShardId);
-    if (!nextChunk)
+    auto foundIt = _cm->getNextChunkOnShard(lookupKey, _thisShardId);
+    if (foundIt.begin() == foundIt.end())
         return false;
 
-    chunk->setMin(nextChunk->getMin());
-    chunk->setMax(nextChunk->getMax());
-
+    const auto& nextChunk = *foundIt.begin();
+    chunk->setMin(nextChunk.getMin());
+    chunk->setMax(nextChunk.getMax());
     return true;
 }
 
@@ -130,23 +134,21 @@ Status CollectionMetadata::checkChunkIsValid(const ChunkType& chunk) const {
         existingChunk.getMax().woCompare(chunk.getMax())) {
         return {ErrorCodes::StaleShardVersion,
                 str::stream() << "Unable to find chunk with the exact bounds "
-                              << chunk.getRange().toString() << " at collection version "
-                              << getCollVersion().toString()
-                              << " found existing chunk: " << existingChunk.toString()};
+                              << ChunkRange(chunk.getMin(), chunk.getMax()).toString()
+                              << " at collection version " << getCollVersion().toString()};
     }
 
     return Status::OK();
 }
-
 boost::optional<ChunkRange> CollectionMetadata::getNextOrphanRange(
-    const RangeMap& receivingChunks, const BSONObj& origLookupKey) const {
+    const RangeMap& chunksMap,
+    const RangeMap& receivingChunks,
+    const BSONObj& origLookupKey) const {
     invariant(isSharded());
 
     const BSONObj maxKey = getMaxKey();
     BSONObj lookupKey = origLookupKey;
 
-    auto chunksMap = getChunks();
-
     while (lookupKey.woCompare(maxKey) < 0) {
         using Its = std::pair<RangeMap::const_iterator, RangeMap::const_iterator>;
 
@@ -207,20 +209,24 @@ boost::optional<ChunkRange> CollectionMetadata::getNextOrphanRange(
     return boost::none;
 }
 
+boost::optional<ChunkRange> CollectionMetadata::getNextOrphanRange(
+    const RangeMap& receivingChunks, const BSONObj& origLookupKey) const {
+    invariant(isSharded());
+    return getNextOrphanRange(getOwnedChunks(), receivingChunks, origLookupKey);
+}
+
 void CollectionMetadata::toBSONChunks(BSONArrayBuilder* builder) const {
     if (!isSharded())
         return;
 
-    _cm->forEachChunk([this, &builder](const auto& chunk) {
+    for (const auto& chunk : _cm->chunks()) {
         if (chunk.getShardId() == _thisShardId) {
             BSONArrayBuilder chunkBB(builder->subarrayStart());
             chunkBB.append(chunk.getMin());
             chunkBB.append(chunk.getMax());
             chunkBB.done();
         }
-
-        return true;
-    });
+    }
 }
 
 }  // namespace mongo
