@@ -1,6 +1,3 @@
-// record_store.h
-
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -37,53 +34,22 @@
 #include "mongo/base/owned_pointer_vector.h"
 #include "mongo/bson/mutable/damage_vector.h"
 #include "mongo/db/exec/collection_scan_common.h"
+#include "mongo/db/namespace_string.h"
 #include "mongo/db/record_id.h"
 #include "mongo/db/storage/record_data.h"
-#include "mongo/db/storage/record_fetcher.h"
 
 namespace mongo {
 
 class CappedCallback;
 class Collection;
-struct CompactOptions;
-struct CompactStats;
 class MAdvise;
-class NamespaceDetails;
 class OperationContext;
-class RecordFetcher;
 
-class RecordStoreCompactAdaptor;
 class RecordStore;
 
 struct ValidateResults;
 class ValidateAdaptor;
 
-/**
- * Allows inserting a Record "in-place" without creating a copy ahead of time.
- */
-class DocWriter {
-public:
-    virtual void writeDocument(char* buf) const = 0;
-    virtual size_t documentSize() const = 0;
-    virtual bool addPadding() const {
-        return true;
-    }
-
-protected:
-    // Can't delete through base pointer.
-    ~DocWriter() = default;
-};
-
-/**
- * @see RecordStore::updateRecord
- */
-class UpdateNotifier {
-public:
-    virtual ~UpdateNotifier() {}
-    virtual Status recordStoreGoingToUpdateInPlace(OperationContext* opCtx,
-                                                   const RecordId& loc) = 0;
-};
-
 /**
  * The data items stored in a RecordStore.
  */
@@ -92,13 +58,6 @@ struct Record {
     RecordData data;
 };
 
-enum ValidateCmdLevel : int {
-    kValidateIndex = 0x01,
-    kValidateRecordStore = 0x02,
-    kValidateFull = 0x03
-};
-
-
 /**
  * Retrieves Records from a RecordStore.
  *
@@ -111,12 +70,12 @@ enum ValidateCmdLevel : int {
  * inside that context. Any cursor acquired inside a transaction is invalid outside
  * of that transaction, instead use the save and restore methods to reestablish the cursor.
  *
- * Any method other than invalidate and the save methods may throw WriteConflictException. If
- * that happens, the cursor may not be used again until it has been saved and successfully
- * restored. If next() or restore() throw a WCE the cursor's position will be the same as before
- * the call (strong exception guarantee). All other methods leave the cursor in a valid state
- * but with an unspecified position (basic exception guarantee). If any exception other than
- * WCE is thrown, the cursor must be destroyed, which is guaranteed not to leak any resources.
+ * Any method other than the save method may throw WriteConflictException. If that happens, the
+ * cursor may not be used again until it has been saved and successfully restored. If next() or
+ * restore() throw a WCE the cursor's position will be the same as before the call (strong exception
+ * guarantee). All other methods leave the cursor in a valid state but with an unspecified position
+ * (basic exception guarantee). If any exception other than WCE is thrown, the cursor must be
+ * destroyed, which is guaranteed not to leak any resources.
  *
  * Any returned unowned BSON is only valid until the next call to any method on this
  * interface.
@@ -146,7 +105,7 @@ public:
 
     /**
      * Moves forward and returns the new data or boost::none if there is no more data.
-     * Continues returning boost::none once it reaches EOF.
+     * Continues returning boost::none once it reaches EOF unlike stl iterators.
      */
     virtual boost::optional<Record> next() = 0;
 
@@ -166,9 +125,10 @@ public:
     /**
      * Recovers from potential state changes in underlying data.
      *
-     * Returns false if it is invalid to continue using this iterator. This usually means that
-     * capped deletes have caught up to the position of this iterator and continuing could
-     * result in missed data.
+     * Returns false if it is invalid to continue using this Cursor. This usually means that
+     * capped deletes have caught up to the position of this Cursor and continuing could
+     * result in missed data. Note that Cursors, unlike iterators can continue to iterate past the
+     * "end"
      *
      * If the former position no longer exists, but it is safe to continue iterating, the
      * following call to next() will return the next closest position in the direction of the
@@ -194,46 +154,12 @@ public:
      * "saved" state, so callers must still call restoreState to use this object.
      */
     virtual void reattachToOperationContext(OperationContext* opCtx) = 0;
-
-    /**
-     * Inform the cursor that this id is being invalidated. Must be called between save and restore.
-     * The opCtx is that of the operation causing the invalidation, not the opCtx using the cursor.
-     *
-     * WARNING: Storage engines other than MMAPv1 should use the default implementation,
-     *          and not depend on this being called.
-     */
-    virtual void invalidate(OperationContext* opCtx, const RecordId& id) {}
-
-    //
-    // RecordFetchers
-    //
-    // Storage engines which do not support document-level locking hold locks at collection or
-    // database granularity. As an optimization, these locks can be yielded when a record needs
-    // to be fetched from secondary storage. If this method returns non-NULL, then it indicates
-    // that the query system layer should yield its locks, following the protocol defined by the
-    // RecordFetcher class, so that a potential page fault is triggered out of the lock.
-    //
-    // Storage engines which support document-level locking need not implement this.
-    //
-    // TODO see if these can be replaced by WriteConflictException.
-    //
-
-    /**
-     * Returns a RecordFetcher if needed for a call to next() or none if unneeded.
-     */
-    virtual std::unique_ptr<RecordFetcher> fetcherForNext() const {
-        return {};
-    }
 };
 
 /**
- * Adds explicit seeking of records. This functionality is separated out from RecordCursor,
- * because some cursors, such as repair cursors, are not required to support seeking.
- *
- * Warning: MMAPv1 cannot detect if RecordIds are valid. Therefore callers should only pass
- * potentially deleted RecordIds to seek methods if they know that MMAPv1 is not the current
- * storage engine. All new storage engines must support detecting the existence of Records.
- *
+ * Adds explicit seeking of records. This functionality is separated out from RecordCursor, because
+ * some cursors are not required to support seeking. All storage engines must support detecting the
+ * existence of Records.
  */
 class SeekableRecordCursor : public RecordCursor {
 public:
@@ -258,30 +184,24 @@ public:
     virtual void saveUnpositioned() {
         save();
     }
-
-    /**
-     * Returns a RecordFetcher if needed to fetch the provided Record or none if unneeded.
-     */
-    virtual std::unique_ptr<RecordFetcher> fetcherForId(const RecordId& id) const {
-        return {};
-    }
 };
 
 /**
- * A RecordStore provides an abstraction used for storing documents in a collection,
- * or entries in an index. In storage engines implementing the KVEngine, record stores
- * are also used for implementing catalogs.
+ * An abstraction used for storing documents in a collection or entries in an index.
+ *
+ * In storage engines implementing the KVEngine, record stores are also used for implementing
+ * catalogs.
  *
  * Many methods take an OperationContext parameter. This contains the RecoveryUnit, with
  * all RecordStore specific transaction information, as well as the LockState. Methods that take
  * an OperationContext may throw a WriteConflictException.
  *
  * This class must be thread-safe for document-level locking storage engines. In addition, for
- * storage engines implementing the KVEngine some methods must be thread safe, see KVCatalog. Only
- * for MMAPv1 is this class not thread-safe.
+ * storage engines implementing the KVEngine some methods must be thread safe, see DurableCatalog.
  */
 class RecordStore {
-    MONGO_DISALLOW_COPYING(RecordStore);
+    RecordStore(const RecordStore&) = delete;
+    RecordStore& operator=(const RecordStore&) = delete;
 
 public:
     RecordStore(StringData ns) : _ns(ns.toString()) {}
@@ -293,10 +213,18 @@ public:
     // name of the RecordStore implementation
     virtual const char* name() const = 0;
 
-    virtual const std::string& ns() const {
+    const std::string& ns() const {
         return _ns;
     }
 
+    void setNs(NamespaceString ns) {
+        _ns = ns.ns();
+    }
+
+    bool isTemp() const {
+        return ns().size() == 0;
+    }
+
     virtual const std::string& getIdent() const = 0;
 
     /**
@@ -306,7 +234,7 @@ public:
     virtual long long dataSize(OperationContext* opCtx) const = 0;
 
     /**
-     * Total number of record in the RecordStore. You may need to cache it, so this call
+     * Total number of records in the RecordStore. You may need to cache it, so this call
      * takes constant time, as it is called often.
      */
     virtual long long numRecords(OperationContext* opCtx) const = 0;
@@ -323,23 +251,35 @@ public:
      * @return total estimate size (in bytes) on stable storage
      */
     virtual int64_t storageSize(OperationContext* opCtx,
-                                BSONObjBuilder* extraInfo = NULL,
+                                BSONObjBuilder* extraInfo = nullptr,
                                 int infoLevel = 0) const = 0;
 
+    /**
+     * @return file bytes available for reuse
+     * A return value of zero can mean either no bytes are available, or that the real value is
+     * unknown.
+     */
+    virtual int64_t freeStorageSize(OperationContext* opCtx) const {
+        return 0;
+    }
+
     // CRUD related
 
     /**
      * Get the RecordData at loc, which must exist.
      *
-     * If unowned data is returned, it is valid until the next modification of this Record or
-     * the lock on this collection is released.
+     * If unowned data is returned, it is only valid until either of these happens:
+     *  - The record is modified
+     *  - The snapshot from which it was obtained is abandoned
+     *  - The lock on the collection is released
      *
-     * In general, prefer findRecord or RecordCursor::seekExact since they can tell you if a
-     * record has been removed.
+     * In general, prefer findRecord or RecordCursor::seekExact since they can tell you if a record
+     * has been removed.
      */
-    virtual RecordData dataFor(OperationContext* opCtx, const RecordId& loc) const {
+    RecordData dataFor(OperationContext* opCtx, const RecordId& loc) const {
         RecordData data;
-        invariant(findRecord(opCtx, loc, &data));
+        invariant(findRecord(opCtx, loc, &data),
+                  str::stream() << "Didn't find RecordId " << loc << " in record store " << ns());
         return data;
     }
 
@@ -352,10 +292,6 @@ public:
      *
      * In general prefer RecordCursor::seekExact since it can avoid copying data in more
      * storageEngines.
-     *
-     * Warning: MMAPv1 cannot detect if RecordIds are valid. Therefore callers should only pass
-     * potentially deleted RecordIds to seek methods if they know that MMAPv1 is not the current
-     * storage engine. All new storage engines must support detecting the existence of Records.
      */
     virtual bool findRecord(OperationContext* opCtx, const RecordId& loc, RecordData* out) const {
         auto cursor = getCursor(opCtx);
@@ -370,77 +306,36 @@ public:
 
     virtual void deleteRecord(OperationContext* opCtx, const RecordId& dl) = 0;
 
-    virtual StatusWith<RecordId> insertRecord(OperationContext* opCtx,
-                                              const char* data,
-                                              int len,
-                                              Timestamp timestamp,
-                                              bool enforceQuota) = 0;
-
+    /**
+     * Inserts the specified records into this RecordStore by copying the passed-in record data and
+     * updates 'inOutRecords' to contain the ids of the inserted records.
+     */
     virtual Status insertRecords(OperationContext* opCtx,
-                                 std::vector<Record>* records,
-                                 std::vector<Timestamp>* timestamps,
-                                 bool enforceQuota) {
-        int index = 0;
-        for (auto& record : *records) {
-            StatusWith<RecordId> res = insertRecord(opCtx,
-                                                    record.data.data(),
-                                                    record.data.size(),
-                                                    (*timestamps)[index++],
-                                                    enforceQuota);
-            if (!res.isOK())
-                return res.getStatus();
-
-            record.id = res.getValue();
-        }
-        return Status::OK();
-    }
+                                 std::vector<Record>* inOutRecords,
+                                 const std::vector<Timestamp>& timestamps) = 0;
 
     /**
-     * Inserts nDocs documents into this RecordStore using the DocWriter interface.
-     *
-     * This allows the storage engine to reserve space for a record and have it built in-place
-     * rather than building the record then copying it into its destination.
-     *
-     * On success, if idsOut is non-null the RecordIds of the inserted records will be written into
-     * it. It must have space for nDocs RecordIds.
+     * A thin wrapper around insertRecords() to simplify handling of single document inserts.
      */
-    virtual Status insertRecordsWithDocWriter(OperationContext* opCtx,
-                                              const DocWriter* const* docs,
-                                              const Timestamp* timestamps,
-                                              size_t nDocs,
-                                              RecordId* idsOut = nullptr) = 0;
-
-    /**
-     * A thin wrapper around insertRecordsWithDocWriter() to simplify handling of single DocWriters.
-     */
-    StatusWith<RecordId> insertRecordWithDocWriter(OperationContext* opCtx,
-                                                   const DocWriter* doc,
-                                                   Timestamp timestamp) {
-        RecordId out;
-        Status status = insertRecordsWithDocWriter(opCtx, &doc, &timestamp, 1, &out);
+    StatusWith<RecordId> insertRecord(OperationContext* opCtx,
+                                      const char* data,
+                                      int len,
+                                      Timestamp timestamp) {
+        std::vector<Record> inOutRecords{Record{RecordId(), RecordData(data, len)}};
+        Status status = insertRecords(opCtx, &inOutRecords, std::vector<Timestamp>{timestamp});
         if (!status.isOK())
             return status;
-        return out;
+        return inOutRecords.front().id;
     }
 
     /**
-     * @param notifier - Only used by record stores which do not support doc-locking. Called only
-     *                   in the case of an in-place update. Called just before the in-place write
-     *                   occurs.
-     * @return Status  - If a document move is required (MMAPv1 only) then a status of
-     *                   ErrorCodes::NeedsDocumentMove will be returned. On receipt of this status
-     *                   no update will be performed. It is the caller's responsibility to:
-     *                     1. Remove the existing document and associated index keys.
-     *                     2. Insert a new document and index keys.
-     *
-     * For capped record stores, the record size will never change.
+     * Updates the record with id 'recordId', replacing its contents with those described by
+     * 'data' and 'len'.
      */
     virtual Status updateRecord(OperationContext* opCtx,
-                                const RecordId& oldLocation,
+                                const RecordId& recordId,
                                 const char* data,
-                                int len,
-                                bool enforceQuota,
-                                UpdateNotifier* notifier) = 0;
+                                int len) = 0;
 
     /**
      * @return Returns 'false' if this record store does not implement
@@ -454,7 +349,7 @@ public:
     /**
      * Updates the record positioned at 'loc' in-place using the deltas described by 'damages'. The
      * 'damages' vector describes contiguous ranges of 'damageSource' from which to copy and apply
-     * byte-level changes to the data.
+     * byte-level changes to the data. Behavior is undefined for calling this on a non-existant loc.
      *
      * @return the updated version of the record. If unowned data is returned, then it is valid
      * until the next modification of this Record or the lock on the collection has been released.
@@ -476,15 +371,6 @@ public:
     virtual std::unique_ptr<SeekableRecordCursor> getCursor(OperationContext* opCtx,
                                                             bool forward = true) const = 0;
 
-    /**
-     * Constructs a cursor over a potentially corrupted store, which can be used to salvage
-     * damaged records. The iterator might return every record in the store if all of them
-     * are reachable and not corrupted.  Returns NULL if not supported.
-     */
-    virtual std::unique_ptr<RecordCursor> getCursorForRepair(OperationContext* opCtx) const {
-        return {};
-    }
-
     /**
      * Constructs a cursor over a record store that returns documents in a randomized order, and
      * allows storage engines to provide a more efficient way of random sampling of a record store
@@ -500,17 +386,6 @@ public:
         return {};
     }
 
-    /**
-     * Returns many RecordCursors that partition the RecordStore into many disjoint sets.
-     * Iterating all returned RecordCursors is equivalent to iterating the full store.
-     */
-    virtual std::vector<std::unique_ptr<RecordCursor>> getManyCursors(
-        OperationContext* opCtx) const {
-        std::vector<std::unique_ptr<RecordCursor>> out(1);
-        out[0] = getCursor(opCtx);
-        return out;
-    }
-
     // higher level
 
 
@@ -537,11 +412,11 @@ public:
     }
 
     /**
-     * Does compact() leave RecordIds alone or can they change.
+     * If compact() supports online compaction.
      *
      * Only called if compactSupported() returns true.
      */
-    virtual bool compactsInPlace() const {
+    virtual bool supportsOnlineCompaction() const {
         MONGO_UNREACHABLE;
     }
 
@@ -549,12 +424,8 @@ public:
      * Attempt to reduce the storage space used by this RecordStore.
      *
      * Only called if compactSupported() returns true.
-     * No RecordStoreCompactAdaptor will be passed if compactsInPlace() returns true.
      */
-    virtual Status compact(OperationContext* opCtx,
-                           RecordStoreCompactAdaptor* adaptor,
-                           const CompactOptions* options,
-                           CompactStats* stats) {
+    virtual Status compact(OperationContext* opCtx) {
         MONGO_UNREACHABLE;
     }
 
@@ -572,15 +443,12 @@ public:
     }
 
     /**
-     * @return OK if the validate run successfully
-     *         OK will be returned even if corruption is found
-     *         deatils will be in result
+     * Performs record store specific validation to ensure consistency of underlying data
+     * structures. If corruption is found, details of the errors will be in the results parameter.
      */
-    virtual Status validate(OperationContext* opCtx,
-                            ValidateCmdLevel level,
-                            ValidateAdaptor* adaptor,
-                            ValidateResults* results,
-                            BSONObjBuilder* output) = 0;
+    virtual void validate(OperationContext* opCtx,
+                          ValidateResults* results,
+                          BSONObjBuilder* output) {}
 
     /**
      * @param scaleSize - amount by which to scale size metrics
@@ -590,20 +458,6 @@ public:
                                    BSONObjBuilder* result,
                                    double scale) const = 0;
 
-    /**
-     * Load all data into cache.
-     * What cache depends on implementation.
-     *
-     * If the underlying storage engine does not support the operation,
-     * returns ErrorCodes::CommandNotSupported
-     *
-     * @param output (optional) - where to put detailed stats
-     */
-    virtual Status touch(OperationContext* opCtx, BSONObjBuilder* output) const {
-        return Status(ErrorCodes::CommandNotSupported,
-                      "this storage engine does not support touch");
-    }
-
     /**
      * Return the RecordId of an oplog entry as close to startingPosition as possible without
      * being higher. If there are no entries <= startingPosition, return RecordId().
@@ -659,40 +513,68 @@ public:
                       "this storage engine does not support updateCappedSize");
     }
 
+    /**
+     * Returns false if the oplog was dropped while waiting for a deletion request.
+     * This should only be called if StorageEngine::supportsOplogStones() is true.
+     * Storage engines supporting oplog stones must implement this function.
+     */
+    virtual bool yieldAndAwaitOplogDeletionRequest(OperationContext* opCtx) {
+        MONGO_UNREACHABLE;
+    }
+
+    /**
+     * This should only be called if StorageEngine::supportsOplogStones() is true.
+     * Storage engines supporting oplog stones must implement this function.
+     */
+    virtual void reclaimOplog(OperationContext* opCtx) {
+        MONGO_UNREACHABLE;
+    }
+
+    /**
+     * This should only be called if StorageEngine::supportsOplogStones() is true.
+     * Storage engines supporting oplog stones must implement this function.
+     * Populates `builder` with various statistics pertaining to oplog stones and oplog truncation.
+     */
+    virtual void getOplogTruncateStats(BSONObjBuilder& builder) const {
+        MONGO_UNREACHABLE;
+    }
+
+    /**
+     * If supported, this method returns the timestamp value for the latest storage engine committed
+     * oplog document. Note that this method will not include uncommitted writes on the input
+     * OperationContext. A new transaction may be created and destroyed to service this call.
+     *
+     * Unsupported RecordStores return the OplogOperationUnsupported error code.
+     */
+    virtual StatusWith<Timestamp> getLatestOplogTimestamp(OperationContext* opCtx) const {
+        return Status(ErrorCodes::OplogOperationUnsupported,
+                      "The current storage engine doesn't support an optimized implementation for "
+                      "getting the latest oplog timestamp.");
+    }
+
+    /**
+     * If supported, this method returns the timestamp value for the earliest storage engine
+     * committed oplog document.
+     *
+     * Unsupported RecordStores return the OplogOperationUnsupported error code.
+     */
+    virtual StatusWith<Timestamp> getEarliestOplogTimestamp(OperationContext* opCtx) {
+        return Status(ErrorCodes::OplogOperationUnsupported,
+                      "The current storage engine doesn't support an optimized implementation for "
+                      "getting the earliest oplog timestamp.");
+    }
+
 protected:
     std::string _ns;
 };
 
-class RecordStoreCompactAdaptor {
-public:
-    virtual ~RecordStoreCompactAdaptor() {}
-    virtual bool isDataValid(const RecordData& recData) = 0;
-    virtual size_t dataSize(const RecordData& recData) = 0;
-    virtual void inserted(const RecordData& recData, const RecordId& newLocation) = 0;
-};
-
 struct ValidateResults {
-    ValidateResults() {
-        valid = true;
-    }
-    bool valid;
+    bool valid = true;
+    bool repaired = false;
+    boost::optional<Timestamp> readTimestamp = boost::none;
     std::vector<std::string> errors;
     std::vector<std::string> warnings;
     std::vector<BSONObj> extraIndexEntries;
     std::vector<BSONObj> missingIndexEntries;
 };
-
-/**
- * This is so when a RecordStore is validating all records
- * it can call back to someone to check if a record is valid.
- * The actual data contained in a Record is totally opaque to the implementation.
- */
-class ValidateAdaptor {
-public:
-    virtual ~ValidateAdaptor() {}
-
-    virtual Status validate(const RecordId& recordId,
-                            const RecordData& recordData,
-                            size_t* dataSize) = 0;
-};
-}
+}  // namespace mongo
